@@ -1,5 +1,8 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
+
+from copy import deepcopy
+import numpy
 from typing import List, Optional
 
 from UM.Math.Matrix import Matrix
@@ -14,10 +17,6 @@ from UM.Logger import Logger
 
 from UM.Scene.SceneNodeDecorator import SceneNodeDecorator
 
-from copy import deepcopy
-import numpy
-
-
 ##  A scene node object.
 #
 #   These objects can hold a mesh and multiple children. Each node has a transformation matrix
@@ -350,19 +349,21 @@ class SceneNode:
     ##  \brief Add a child to this node and set it's parent as this node.
     #   \params scene_node SceneNode to add.
     def addChild(self, scene_node: "SceneNode"):
-        if scene_node not in self._children:
-            scene_node.transformationChanged.connect(self.transformationChanged)
-            scene_node.childrenChanged.connect(self.childrenChanged)
-            scene_node.meshDataChanged.connect(self.meshDataChanged)
+        if scene_node in self._children:
+            return
 
-            self._children.append(scene_node)
-            self._resetAABB()
-            self.childrenChanged.emit(self)
+        scene_node.transformationChanged.connect(self.transformationChanged)
+        scene_node.childrenChanged.connect(self.childrenChanged)
+        scene_node.meshDataChanged.connect(self.meshDataChanged)
 
-            if not scene_node._parent is self:
-                scene_node._parent = self
-                scene_node._transformChanged()
-                scene_node.parentChanged.emit(self)
+        self._children.append(scene_node)
+        self._resetAABB()
+        self.childrenChanged.emit(self)
+
+        if not scene_node._parent is self:
+            scene_node._parent = self
+            scene_node._transformChanged()
+            scene_node.parentChanged.emit(self)
 
     ##  \brief remove a single child
     #   \param child Scene node that needs to be removed.
@@ -607,7 +608,10 @@ class SceneNode:
 
     ##  Get whether this SceneNode is enabled, that is, it can be modified in any way.
     def isEnabled(self) -> bool:
-        return self._enabled
+        if self._parent is not None and self._enabled:
+            return self._parent.isEnabled()
+        else:
+            return self._enabled
 
     ##  Set whether this SceneNode is enabled.
     #   \param enable True if this object should be enabled, False if not.
